/*
 * FeRAM_CY15B108.c
 *  Created on: 20.03.2015 as FRAM_RS2MT_SPI.c
 *      Author: DL9MWE
 *  21.04.2019
 *  17.02.2023 adaptiert fr 8MBit FeRAM CY15B108
 */
#include "main.h"

// initialize and check FeRAMs
unsigned int uiFeRAM_CY15B108_init() {
    vSPI_init();
    // Chip Selects (CS1 und CS2) sind Ausgnge vom MSP430 und werden auf 1 gesetzt
    PORT_SPI_SEL &= ~(PORTFeRAM_CS1_PIN+PORTFeRAM_CS2_PIN);
    PORT_SPI_SEL2 &= ~(PORTFeRAM_CS1_PIN+PORTFeRAM_CS2_PIN);
    PORT_SPI_DIR |= (PORTFeRAM_CS1_PIN+PORTFeRAM_CS2_PIN);
    PORT_SPI_OUT |= (PORTFeRAM_CS1_PIN+PORTFeRAM_CS2_PIN);

    // Prsenz der Chips verifizieren
    if(FeRAM_check_deviceID(PORTFeRAM_CS1_PIN)) {       // den ersten Chip prfen
        if(FeRAM_check_deviceID(PORTFeRAM_CS2_PIN)) {   // den zweiten Chip prfen
            return 1;                                   // success
        }
    }
    return 0;                                           // error
}

// prfen, ob ein Chip ansprechbar ist
//  return 0, falls nicht
unsigned int FeRAM_check_deviceID(unsigned int CS_PIN) {
    unsigned char deviceID[9];
    PORT_SPI_OUT &= ~CS_PIN;                // Start: entspr. CS-Pin auf 0
    ucSPI_Byte(FeRAM_RDID);                 // send command: Read Device ID
    vSPI_read_multiplebytes(9, deviceID);   // read 9 Bytes of answer
    PORT_SPI_OUT |= CS_PIN;                 // fertig: entspr. CS-Pin auf 1

    // Device ID fr den Chip CYB108QN-40SX soll 0x7F7F7F7F7F7FC22E03 ergeben (Datenblatt Seite 24)
    unsigned char soll_deviceID[9] = "\x03\x2E\xC2\x7F\x7F\x7F\x7F\x7F\x7F";    // in richtiger Reihenfolge...
    unsigned int ii=9;
    while(--ii) {
        if(deviceID[ii] != soll_deviceID[ii])  return 0;
    }
    return 1;
}

// schreibt mehrere Bytes ab der Adresse addr in das FeRAM. addr deckt den Bereich vom Anfang des ersten FeRAMs bis zum Ende des zweiten FeRAMs ab.
// Rckgabe: die nchste Schreibadresse, im Fehlerfall (berlauf) 0UL
unsigned long ulFeRAM_CY15B108_writebytes(unsigned long addr, unsigned char *txData, unsigned int anzahl) {
    unsigned char SPI_CS_pin;
    unsigned long wr_addr;              // tatschliche Schreibadresse im ersten oder zweiten FeRAM

    if(addr <= FeRAM_maxAddr) {         // erstes FeRAM
        wr_addr = addr;
        SPI_CS_pin = PORTFeRAM_CS1_PIN;
    }
    else {                              // zweites FeRAM
        wr_addr = addr-FeRAM_maxAddr-1;
        SPI_CS_pin = PORTFeRAM_CS2_PIN;
    }
    PORT_SPI_OUT &= ~SPI_CS_pin;        // _CS auf 0
    ucSPI_Byte(FeRAM_WREN);             // Write enable command
    PORT_SPI_OUT |= SPI_CS_pin;         // _CS auf 1
    _NOP();
    PORT_SPI_OUT &= ~SPI_CS_pin;        // _CS auf 0
    ucSPI_Byte(FeRAM_WRITE);
    ucSPI_Byte((unsigned char)(wr_addr>>16));
    ucSPI_Byte((unsigned char)(wr_addr>>8));
    ucSPI_Byte((unsigned char) wr_addr);

    unsigned int index=0;
    while(index < anzahl) {
        ucSPI_Byte(txData[index++]);
        ++addr;
        if(++wr_addr > FeRAM_maxAddr) {           // Speicherende des jeweiligen FeRAMs erreicht?
            PORT_SPI_OUT |= SPI_CS_pin;           // _CS des FeRAMs auf 1
            if(SPI_CS_pin == PORTFeRAM_CS2_PIN) { // Ende des zweiten FeRAMs?
                addr = 0UL;                       // Fehler: Speicherberlauf!
                break;
            }
            // sonst im zweiten FeRAM weiterschreiben
            addr = ulFeRAM_CY15B108_writebytes(wr_addr, &txData[index], anzahl-index);
            break;
        }
    }
    PORT_SPI_OUT |= PORTFeRAM_CS1_PIN + PORTFeRAM_CS2_PIN;    // beide _CS auf 1
    return addr;
}

// liest mehrere Bytes ab der Adresse addr aus dem FeRAM. addr deckt den Bereich vom Anfang des ersten FeRAMs bis zum Ende des zweiten FeRAMs ab.
// Rckgabe: die nchste Leseadresse, im Fehlerfall (berlauf) 0UL
unsigned long ulFeRAM_CY15B108_readbytes(unsigned long addr, unsigned char *rxData, unsigned int anzahl) {
    unsigned int SPI_CS_pin;
    unsigned long rd_addr;          // tatschliche Leseadresse im ersten oder zweiten FeRAM

    if(addr <= FeRAM_maxAddr) {     // erstes FeRAM
        rd_addr = addr;
        SPI_CS_pin = PORTFeRAM_CS1_PIN;
    }
    else {                          // zweites FeRAM
        rd_addr = addr-FeRAM_maxAddr-1;
        SPI_CS_pin = PORTFeRAM_CS2_PIN;
    }
    PORT_SPI_OUT &= ~SPI_CS_pin;    // _CS auf 0
    ucSPI_Byte(FeRAM_READ);
    ucSPI_Byte((unsigned char)(rd_addr>>16));
    ucSPI_Byte((unsigned char)(rd_addr>>8));
    ucSPI_Byte((unsigned char) rd_addr);

    unsigned int index=0;
    while(index < anzahl) {
        rxData[index++] = ucSPI_Byte(0x00);             // read only, dummy write
        ++addr;
        if(++rd_addr > FeRAM_maxAddr) {                 // Speicherende des jeweiligen FeRAMs erreicht?
            PORT_SPI_OUT |= SPI_CS_pin;                 // _CS auf 1
            if(SPI_CS_pin == PORTFeRAM_CS2_PIN) {       // Ende des zweiten FeRAMs
                addr = 0UL;                             // Fehler: Speicherberlauf!
                break;
            }
            // sonst im zweiten FeRAM weiterlesen
            addr = ulFeRAM_CY15B108_readbytes(rd_addr, &rxData[index], anzahl-index);
            break;
        }
    }
    PORT_SPI_OUT |= PORTFeRAM_CS1_PIN + PORTFeRAM_CS2_PIN;            // beide _CS auf 1
    return addr;
}

// Ende FeRAM_CY15B108.c
